/*
 * Copyright (C) 1995 Phil Servita (meister@ftp.com)
 * portions derived from the Bellcore S/Key 
 *
/* Stand-alone program for computing responses to OTP challenges.
 * Takes the iteration count and seed as command line args, prompts
 * for the user's key, and produces both word and hex format responses.
 *
 * Usage example:
 *	>otpkey -H md4 99 foo
 *	Enter password:
 *	CHIC COD PAD FORM FEEL RITE
 *	>
 */
#include <stdio.h>

#ifdef USE_STDLIB_H
# include <stdlib.h>
#endif

#include <sys/types.h>
#include <string.h>

#ifdef	__MSDOS__
#include <dos.h>
#else	/* Assume BSD unix */
#include <fcntl.h>
#endif

/* #include "md4.h" */
#include "otp.h"
#include "ps_hdr.h"

char *_otp_readpass();
void usage __ARGS((char *));

#ifndef STDLIB_HAS_GETOPT
int getopt __ARGS((int, char **, char *));
#endif

extern int optind;
extern char *optarg;
int dash_P = 0;

int main (argc,argv)
        int argc;
        char *argv [];
{
	int n,cnt,i;
	char passwd[256],passwd2[256];
	char key[8];
	char *seed;
	char seedbuf [32];
	char buf[33];
	char *slash;
        
        OTPkeycrunchfcn keycrunch_fcn;
        OTPiteratefcn   iterate_fcn;

        char hash_alg [64];
    
        strcpy (hash_alg, "md5");

	cnt = 1;
	while ((i = getopt (argc,argv,"H:n:P?")) != EOF)
        {
		switch (i)
                {
                        case 'n':
		             cnt = atoi(optarg);
                             break;

                        case 'H': 
                             fprintf (stdout, "Using alg %s\n", optarg);
                             strcpy (hash_alg, optarg);
                             break;

                        case 'P':
                             dash_P = 1;
                             break;
			     
                        case '?':
                             usage (argv[0]);
			     exit (0);
                             break;			     
                }
        }
        if (argc == optind)
        {   printf ("Enter Sequence Count: ");
            fgets (seedbuf, 31, stdin);
            n = atoi (seedbuf);
            printf ("\nEnter Seed: ");
            fgets (seedbuf, 31, stdin);
	    _otp_rip (seedbuf);
	    seed = seedbuf;
        }
        else
        {
            /* could be in the form <number>/<seed> */
            if (argc <= optind + 1)
            {
                /*look for / in it */
                if (argc <= optind)
                {
                    usage (argv[0]);
                    return 1;
                }

                slash = strchr (argv[optind], '/');
                if (slash == NULL)
                {
                    usage (argv[0]);
                    return 1;
                }
                *slash++ = '\0';
                seed = slash;

                if ((n = atoi (argv [optind])) < 0)
                {
                    fprintf(stderr,"%s not positive\n",argv[optind]);
                    usage (argv[0]);
                    return 1;
                }
            }
            else 
            {
                if ((n = atoi (argv [optind])) < 0)
                {
                    fprintf (stderr, "%s not positive\n", argv [optind]);
                    usage (argv[0]);
                    return 1;
                }
                seed = argv [++optind];
            }
        }

        /*
         * Based on the algorithm specified, get the right keycrunch 
         * and iterate functions.
         */
        keycrunch_fcn = OTP_GetKeyCrunchFcn (hash_alg);
        if (!keycrunch_fcn)
        {   fprintf (stderr, "Unknown algorithm type %s\n", hash_alg);
            exit (-1);
        }
        iterate_fcn = OTP_GetIterateFcn (hash_alg);
        if (!iterate_fcn)
        {   fprintf (stderr, "Unknown algorithm type %s\n", hash_alg);
            exit (-1);
        }

#ifndef MSDOS        
        fprintf (stderr, "Reminder - Do not use otpkey while logged in via telnet or dial-in.\n");
        fprintf (stderr, "If you are not on the system console, or logged in via\n");
        fprintf (stderr, "an encrypted session, do not enter your OTP password here..\n");
        fprintf (stderr, "Find an OTP generator somewhere else, such as a PC.\n");
#endif

        /* Get user's secret password */
        for (;;)
        {
             fprintf (stderr,"\nEnter secret password: ");
             _otp_readpass (passwd,sizeof(passwd));
#ifndef ASK_PASSWD_VERIFY
             break;
#else
             fprintf (stderr, "Repeat secret password (to avoid typos): ");
             _otp_readpass (passwd2, sizeof (passwd));
             if (strcmp (passwd, passwd2) == 0) 
                 break;
             fprintf (stderr, "Sorry, entries do not match.\n");
#endif
        }

        /* Crunch seed and password into starting key */
        if (OTP_KeyCrunch (key, seed, passwd, keycrunch_fcn) != 0)
        {
            fprintf(stderr,"%s: key crunch failed\n",argv[0]);
            return 1;
        }
        if (cnt == 1)
        {
            while (n-- != 0)
                   OTP_IterateHash (key, iterate_fcn);
            printf("%s\n",OTP_btoe (buf,key));
#ifdef HEXIN
            printf("%s\n",OTP_put8 (buf,key));
#endif
        }
        else 
        {
            char **PS = ps_hdr;
            if (dash_P)
            {   while (*PS)
                {      fprintf (stdout, "%s\n", *PS);
                       PS++;
                }
                fprintf (stdout, "\n");
            }

            for (i=0; i <= (n - cnt); i++)
                 OTP_IterateHash (key, iterate_fcn);
            for (/* nop */ ; i <= n ; i++)
            {
#ifdef HEXIN
                 printf ("%d: %-29s", i, OTP_btoe (buf, key));
                 printf ("  %s\n", OTP_put8 (buf, key));
#else
                 printf ("%d: %-29s\n", i, OTP_btoe (buf, key));
#endif
                 OTP_IterateHash (key, iterate_fcn);                
            }
        }
        return 0;
}

void usage (s)
    char *s;
{
    fprintf (stderr, "Usage: %s [-H md4|md5|sha1] [-n count] [-P] <sequence #>[/] <key> \n",s);
    fprintf (stderr, "Bugs to phil servita <meister@ftp.com>\n");
}

